@@ -15,17 +15,117 @@
 #    under the License.
 
 """Ironic specific exceptions list."""
-
+import collections
 from http import client as http_client
+import json
 
-from ironic_lib.exception import IronicException
+from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import excutils
 
 from ironic.common.i18n import _
 
 LOG = logging.getLogger(__name__)
 
 
+CONF = cfg.CONF
+
+
+def _ensure_exception_kwargs_serializable(exc_class_name, kwargs):
+    """Ensure that kwargs are serializable
+
+    Ensure that all kwargs passed to exception constructor can be passed over
+    RPC, by trying to convert them to JSON, or, as a last resort, to string.
+    If it is not possible, unserializable kwargs will be removed, letting the
+    receiver handle the exception string as it is configured to.
+
+    :param exc_class_name: a IronicException class name.
+    :param kwargs: a dictionary of keyword arguments passed to the exception
+        constructor.
+    :returns: a dictionary of serializable keyword arguments.
+    """
+    serializers = [(json.dumps, _('when converting to JSON')),
+                   (str, _('when converting to string'))]
+    exceptions = collections.defaultdict(list)
+    serializable_kwargs = {}
+    for k, v in kwargs.items():
+        for serializer, msg in serializers:
+            try:
+                serializable_kwargs[k] = serializer(v)
+                exceptions.pop(k, None)
+                break
+            except Exception as e:
+                exceptions[k].append(
+                    '(%(serializer_type)s) %(e_type)s: %(e_contents)s' %
+                    {'serializer_type': msg, 'e_contents': e,
+                     'e_type': e.__class__.__name__})
+    if exceptions:
+        LOG.error("One or more arguments passed to the %(exc_class)s "
+                  "constructor as kwargs can not be serialized. The "
+                  "serialized arguments: %(serialized)s. These "
+                  "unserialized kwargs were dropped because of the "
+                  "exceptions encountered during their "
+                  "serialization:\n%(errors)s",
+                  dict(errors=';\n'.join("%s: %s" % (k, '; '.join(v))
+                                         for k, v in exceptions.items()),
+                       exc_class=exc_class_name,
+                       serialized=serializable_kwargs))
+        # We might be able to actually put the following keys' values into
+        # format string, but there is no guarantee, drop it just in case.
+        for k in exceptions:
+            del kwargs[k]
+    return serializable_kwargs
+
+
+class IronicException(Exception):
+    """Base Ironic Exception
+
+    To correctly use this class, inherit from it and define
+    a '_msg_fmt' property. That _msg_fmt will get printf'd
+    with the keyword arguments provided to the constructor.
+
+    If you need to access the message from an exception you should use
+    str(exc)
+
+    """
+
+    _msg_fmt = _("An unknown exception occurred.")
+    code = 500
+    headers = {}
+    safe = False
+
+    def __init__(self, message=None, **kwargs):
+        self.kwargs = _ensure_exception_kwargs_serializable(
+            self.__class__.__name__, kwargs)
+
+        if 'code' not in self.kwargs:
+            try:
+                self.kwargs['code'] = self.code
+            except AttributeError:
+                pass
+        else:
+            self.code = int(kwargs['code'])
+
+        if not message:
+            try:
+                message = self._msg_fmt % kwargs
+
+            except Exception:
+                with excutils.save_and_reraise_exception() as ctxt:
+                    # kwargs doesn't match a variable in the message
+                    # log the issue and the kwargs
+                    prs = ', '.join('%s=%s' % pair for pair in kwargs.items())
+                    LOG.exception('Exception in string format operation '
+                                  '(arguments %s)', prs)
+                    if not CONF.errors.fatal_exception_format_errors:
+                        # at least get the core message out if something
+                        # happened
+                        message = self._msg_fmt
+                        ctxt.reraise = False
+
+        super(IronicException, self).__init__(message)
+
+
 class NotAuthorized(IronicException):
     _msg_fmt = _("Not authorized.")
     code = http_client.FORBIDDEN
@@ -716,6 +816,22 @@ class InvalidDeployTemplate(Invalid):
     _msg_fmt = _("Deploy template invalid: %(err)s.")
 
 
+class RunbookDuplicateName(Conflict):
+    _msg_fmt = _("A runbook with name %(name)s already exists.")
+
+
+class RunbookAlreadyExists(Conflict):
+    _msg_fmt = _("A runbook with UUID %(uuid)s already exists.")
+
+
+class RunbookNotFound(NotFound):
+    _msg_fmt = _("Runbook %(runbook)s could not be found.")
+
+
+class InvalidRunbook(Invalid):
+    _msg_fmt = _("Runbook invalid: %(err)s.")
+
+
 class InvalidKickstartTemplate(Invalid):
     _msg_fmt = _("The kickstart template is missing required variables")
 
@@ -845,8 +961,7 @@ class ImageRefIsARedirect(IronicException):
 
     def __init__(self, image_ref=None, redirect_url=None, msg=None):
         self.redirect_url = redirect_url
-        # Kwargs are expected by ironic_lib's IronicException to convert
-        # the message.
+        # Kwargs are expected by IronicException to convert the message.
         super(ImageRefIsARedirect, self).__init__(
             message=msg,
             image_ref=image_ref,
@@ -915,3 +1030,35 @@ class ImageChecksumFileReadFailure(InvalidImage):
     _msg_fmt = _("Failed to read the file from local storage "
                  "to perform a checksum operation.")
     code = http_client.SERVICE_UNAVAILABLE
+
+
+class ParentNodeLocked(Conflict):
+    _msg_fmt = _("Node %(node)s parent_node %(parent)s is presently locked "
+                 "and we are unable to perform any action on it at this "
+                 "time. Please retry after the current operation is "
+                 "completed.")
+
+
+class ChildNodeLocked(Conflict):
+    _msg_fmt = _("Node %(node)s child_node %(child)s is presently locked "
+                 "and we are unable to perform any action on it at this "
+                 "time. Please retry after the current operation is "
+                 "completed.")
+
+
+class MetricsNotSupported(IronicException):
+    _msg_fmt = _("Metrics action is not supported. You may need to "
+                 "adjust the [metrics] section in ironic.conf.")
+
+
+class ServiceLookupFailure(IronicException):
+    _msg_fmt = _("Cannot find %(service)s service through multicast.")
+
+
+class ServiceRegistrationFailure(IronicException):
+    _msg_fmt = _("Cannot register %(service)s service: %(error)s")
+
+
+class Unauthorized(IronicException):
+    code = http_client.UNAUTHORIZED
+    headers = {'WWW-Authenticate': 'Basic realm="Baremetal API"'}
