@@ -1347,6 +1347,101 @@ class Connection(object, metaclass=abc.ABCMeta):
         :returns: A list of deploy templates.
         """
 
+    @abc.abstractmethod
+    def create_runbook(self, values):
+        """Create a runbook.
+
+        :param values: A dict describing the runbook. For example:
+
+                     ::
+
+                      {
+                       'uuid': uuidutils.generate_uuid(),
+                       'name': 'CUSTOM_DT1',
+                      }
+        :raises: RunbookDuplicateName if a runbook with the same
+            name exists.
+        :raises: RunbookAlreadyExists if a runbook with the same
+            UUID exists.
+        :returns: A runbook.
+        """
+
+    @abc.abstractmethod
+    def update_runbook(self, runbook_id, values):
+        """Update a runbook.
+
+        :param runbook_id: ID of the runbook to update.
+        :param values: A dict describing the runbook. For example:
+
+                     ::
+
+                      {
+                       'uuid': uuidutils.generate_uuid(),
+                       'name': 'CUSTOM_DT1',
+                      }
+        :raises: RunbookDuplicateName if a runbook with the same
+            name exists.
+        :raises: RunbookNotFound if the runbook does not exist.
+        :returns: A runbook.
+        """
+
+    @abc.abstractmethod
+    def destroy_runbook(self, runbook_id):
+        """Destroy a runbook.
+
+        :param runbook_id: ID of the runbook to destroy.
+        :raises: RunbookNotFound if the runbook does not exist.
+        """
+
+    @abc.abstractmethod
+    def get_runbook_by_id(self, runbook_id):
+        """Retrieve a runbook by ID.
+
+        :param runbook_id: ID of the runbook to retrieve.
+        :raises: RunbookNotFound if the runbook does not exist.
+        :returns: A runbook.
+        """
+
+    @abc.abstractmethod
+    def get_runbook_by_uuid(self, runbook_uuid):
+        """Retrieve a runbook by UUID.
+
+        :param runbook_uuid: UUID of the runbook to retrieve.
+        :raises: RunbookNotFound if the runbook does not exist.
+        :returns: A runbook.
+        """
+
+    @abc.abstractmethod
+    def get_runbook_by_name(self, runbook_name):
+        """Retrieve a runbook by name.
+
+        :param runbook_name: name of the runbook to retrieve.
+        :raises: RunbookNotFound if the runbook does not exist.
+        :returns: A runbook.
+        """
+
+    @abc.abstractmethod
+    def get_runbook_list(self, limit=None, marker=None, filters=None,
+                         sort_key=None, sort_dir=None):
+        """Retrieve a list of runbooks.
+
+        :param limit: Maximum number of runbooks to return.
+        :param marker: The last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted.
+        :param sort_dir: Direction in which results should be sorted.
+                         (asc, desc)
+        :returns: A list of runbooks.
+        """
+
+    @abc.abstractmethod
+    def get_runbook_list_by_names(self, names):
+        """Return a list of runbooks with one of a list of names.
+
+        :param names: List of names to filter by.
+        :returns: A list of runbooks.
+        """
+
     @abc.abstractmethod
     def create_node_history(self, values):
         """Create a new history record.
@@ -1419,7 +1514,7 @@ class Connection(object, metaclass=abc.ABCMeta):
     def bulk_delete_node_history_records(self, node_id, limit):
         """Utility method to bulk delete node history entries.
 
-        :param entires: A list of node history entriy id's to be
+        :param entries: A list of node history entry id's to be
                         queried for deletion.
         """
 
@@ -1461,7 +1556,8 @@ class Connection(object, metaclass=abc.ABCMeta):
         :returns: list of dicts containing shard names and count
         """
 
-    @abc.abstractclassmethod
+    @classmethod
+    @abc.abstractmethod
     def create_firmware_component(self, values):
         """Create a FirmwareComponent record for a given node.
 
@@ -1481,7 +1577,8 @@ class Connection(object, metaclass=abc.ABCMeta):
             records already exists.
         """
 
-    @abc.abstractclassmethod
+    @classmethod
+    @abc.abstractmethod
     def update_firmware_component(self, node_id, component, values):
         """Update a FirmwareComponent record.
 
@@ -1513,7 +1610,8 @@ class Connection(object, metaclass=abc.ABCMeta):
             is not found.
         """
 
-    @abc.abstractclassmethod
+    @classmethod
+    @abc.abstractmethod
     def get_firmware_component_list(self, node_id):
         """Retrieve a list Firmware Components of a given node.
 
@@ -1521,3 +1619,21 @@ class Connection(object, metaclass=abc.ABCMeta):
         :returns: A list of FirmwareComponent objects.
         :raises: NodeNotFound if the node is not found.
         """
+
+    @abc.abstractclassmethod
+    def get_child_node_ids_by_parent_uuid(self, node_uuid,
+                                          exclude_dedicated_power=False):
+        """Retrieve a list of child node IDs for a given parent UUID.
+
+        This is an "internal" method, intended for use with power
+        management logic to facilitate power actions upon nodes,
+        where we obtain a list of nodes which requires additional
+        actions, and return *only* the required node IDs in order
+        to launch new tasks for power management activities.
+
+        :param node_uuid: The uuid of the parent node, in order to
+            directly match the "parent_node" field.
+        :param exclude_dedicated_power: Boolean, False, if the list
+            should include child nodes with their own power supplies.
+        :returns: A list of tuples.
+        """
