@@ -17,13 +17,12 @@
 import os
 import re
 
-from ironic_lib import metrics_utils
-from ironic_lib import utils as il_utils
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
 from oslo_utils import strutils
 
+from ironic.common import async_steps
 from ironic.common import checksum_utils
 from ironic.common import context
 from ironic.common import exception
@@ -33,6 +32,7 @@ from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import images
 from ironic.common import keystone
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import utils as manager_utils
@@ -771,7 +771,7 @@ def tear_down_inband_cleaning(task, manage_boot=True):
     node = task.node
     cleaning_failure = (node.fault == faults.CLEAN_FAILURE)
 
-    if not (fast_track or cleaning_failure):
+    if not (fast_track or cleaning_failure or node.disable_power_off):
         manager_utils.node_power_action(task, states.POWER_OFF)
 
     if manage_boot:
@@ -780,8 +780,11 @@ def tear_down_inband_cleaning(task, manage_boot=True):
     power_state_to_restore = manager_utils.power_on_node_if_needed(task)
     task.driver.network.remove_cleaning_network(task)
     if not (fast_track or cleaning_failure):
-        manager_utils.restore_power_state_if_needed(
-            task, power_state_to_restore)
+        if node.disable_power_off:
+            manager_utils.node_power_action(task, states.REBOOT)
+        else:
+            manager_utils.restore_power_state_if_needed(
+                task, power_state_to_restore)
 
 
 def prepare_inband_service(task):
@@ -796,24 +799,23 @@ def prepare_inband_service(task):
     :raises: any boot interface's prepare_ramdisk exceptions.
     :returns: Returns states.SERVICEWAIT
     """
-    manager_utils.node_power_action(task, states.POWER_OFF)
-    # NOTE(TheJulia): Revealing that the power is off at any time can
-    # cause external power sync to decide that the node must be off.
-    # This may result in a post-rescued instance being turned off
-    # unexpectedly after rescue has started.
-    # TODO(TheJulia): Once we have power/state callbacks to nova,
-    # the reset of the power_state can be removed.
-    task.node.power_state = states.POWER_ON
-    task.node.save()
+    with driver_utils.power_off_and_on(task):
+        # NOTE(TheJulia): Revealing that the power is off at any time can
+        # cause external power sync to decide that the node must be off.
+        # This may result in a post-rescued instance being turned off
+        # unexpectedly after rescue has started.
+        # TODO(TheJulia): Once we have power/state callbacks to nova,
+        # the reset of the power_state can be removed.
+        task.node.power_state = states.POWER_ON
+        task.node.save()
 
-    task.driver.boot.clean_up_instance(task)
-    with manager_utils.power_state_for_network_configuration(task):
-        task.driver.network.unconfigure_tenant_networks(task)
-        task.driver.network.add_servicing_network(task)
-    if CONF.agent.manage_agent_boot:
-        # prepare_ramdisk will set the boot device
-        prepare_agent_boot(task)
-    manager_utils.node_power_action(task, states.POWER_ON)
+        task.driver.boot.clean_up_instance(task)
+        with manager_utils.power_state_for_network_configuration(task):
+            task.driver.network.unconfigure_tenant_networks(task)
+            task.driver.network.add_servicing_network(task)
+        if CONF.agent.manage_agent_boot:
+            # prepare_ramdisk will set the boot device
+            prepare_agent_boot(task)
 
     return states.SERVICEWAIT
 
@@ -837,26 +839,22 @@ def tear_down_inband_service(task):
     node = task.node
     service_failure = (node.fault == faults.SERVICE_FAILURE)
 
-    if not service_failure:
+    if not service_failure and not node.disable_power_off:
         manager_utils.node_power_action(task, states.POWER_OFF)
 
     task.driver.boot.clean_up_ramdisk(task)
 
-    power_state_to_restore = manager_utils.power_on_node_if_needed(task)
-
     if not service_failure:
-        manager_utils.restore_power_state_if_needed(
-            task, power_state_to_restore)
-
         with manager_utils.power_state_for_network_configuration(task):
             task.driver.network.remove_servicing_network(task)
             task.driver.network.configure_tenant_networks(task)
-        task.driver.boot.prepare_instance(task)
-        manager_utils.restore_power_state_if_needed(
-            task, power_state_to_restore)
 
-        # Change the task instead of return the state.
-        task.process_event('done')
+        task.driver.boot.prepare_instance(task)
+        # prepare_instance does not power on the node, the deploy interface is
+        # normally responsible for that.
+        next_state = (states.REBOOT if task.node.disable_power_off
+                      else states.POWER_ON)
+        manager_utils.node_power_action(task, next_state)
 
 
 def get_image_instance_info(node):
@@ -1132,7 +1130,7 @@ def destroy_images(node_uuid):
 
     :param node_uuid: the uuid of the ironic node.
     """
-    il_utils.unlink_without_raise(_get_image_file_path(node_uuid))
+    utils.unlink_without_raise(_get_image_file_path(node_uuid))
     utils.rmtree_without_raise(_get_image_dir_path(node_uuid))
     InstanceImageCache().clean_up()
 
@@ -1149,7 +1147,7 @@ def compute_image_checksum(image_path, algorithm='md5'):
 
 def remove_http_instance_symlink(node_uuid):
     symlink_path = _get_http_image_symlink_file_path(node_uuid)
-    il_utils.unlink_without_raise(symlink_path)
+    utils.unlink_without_raise(symlink_path)
 
 
 def destroy_http_instance_images(node):
@@ -1232,13 +1230,13 @@ def _validate_image_url(node, url, secret=False, inspect_image=None,
             expected_format=expected_format)
         # NOTE(TheJulia): We explicitly delete this file because it has no use
         # in the cache after this point.
-        il_utils.unlink_without_raise(image_path)
+        utils.unlink_without_raise(image_path)
         image_info['disk_format'] = img_format
     return image_info
 
 
 def _cache_and_convert_image(task, instance_info, image_info=None):
-    """Cache an image locally and covert it to RAW if needed."""
+    """Cache an image locally and convert it to RAW if needed."""
     # Ironic cache and serve images from httpboot server
     force_raw = direct_deploy_should_convert_raw_image(task.node)
 
@@ -1631,74 +1629,9 @@ parse_instance_info_capabilities = (
     utils.parse_instance_info_capabilities
 )
 
-
-def get_async_step_return_state(node):
-    """Returns state based on operation (cleaning/deployment) being invoked
-
-    :param node: an ironic node object.
-    :returns: states.CLEANWAIT if cleaning operation in progress
-              or states.DEPLOYWAIT if deploy operation in progress.
-    """
-    return states.CLEANWAIT if node.clean_step else states.DEPLOYWAIT
-
-
-def _check_agent_token_prior_to_agent_reboot(node):
-    """Removes the agent token if it was not pregenerated.
-
-    Removal of the agent token in cases where it is not pregenerated
-    is a vital action prior to rebooting the agent, as without doing
-    so the agent will be unable to establish communication with
-    the ironic API after the reboot. Effectively locking itself out
-    as in cases where the value is not pregenerated, it is not
-    already included in the payload and must be generated again
-    upon lookup.
-
-    :param node: The Node object.
-    """
-    if not node.driver_internal_info.get('agent_secret_token_pregenerated',
-                                         False):
-        node.del_driver_internal_info('agent_secret_token')
-
-
-def set_async_step_flags(node, reboot=None, skip_current_step=None,
-                         polling=None):
-    """Sets appropriate reboot flags in driver_internal_info based on operation
-
-    :param node: an ironic node object.
-    :param reboot: Boolean value to set for node's driver_internal_info flag
-        cleaning_reboot or deployment_reboot based on cleaning or deployment
-        operation in progress. If it is None, corresponding reboot flag is
-        not set in node's driver_internal_info.
-    :param skip_current_step: Boolean value to set for node's
-        driver_internal_info flag skip_current_clean_step or
-        skip_current_deploy_step based on cleaning or deployment operation
-        in progress. If it is None, corresponding skip step flag is not set
-        in node's driver_internal_info.
-    :param polling: Boolean value to set for node's driver_internal_info flag
-        deployment_polling or cleaning_polling. If it is None, the
-        corresponding polling flag is not set in the node's
-        driver_internal_info.
-    """
-    if node.clean_step:
-        reboot_field = 'cleaning_reboot'
-        skip_field = 'skip_current_clean_step'
-        polling_field = 'cleaning_polling'
-    else:
-        reboot_field = 'deployment_reboot'
-        skip_field = 'skip_current_deploy_step'
-        polling_field = 'deployment_polling'
-
-    if reboot is not None:
-        node.set_driver_internal_info(reboot_field, reboot)
-        if reboot:
-            # If rebooting, we must ensure that we check and remove
-            # an agent token if necessary.
-            _check_agent_token_prior_to_agent_reboot(node)
-    if skip_current_step is not None:
-        node.set_driver_internal_info(skip_field, skip_current_step)
-    if polling is not None:
-        node.set_driver_internal_info(polling_field, polling)
-    node.save()
+# NOTE(dtantsur): backward compatibility, do not use
+get_async_step_return_state = async_steps.get_return_state
+set_async_step_flags = async_steps.set_node_flags
 
 
 def prepare_agent_boot(task):
@@ -1722,14 +1655,31 @@ def reboot_to_finish_step(task, timeout=None):
     disable_ramdisk = task.node.driver_internal_info.get(
         'cleaning_disable_ramdisk')
     if not disable_ramdisk:
-        if manager_utils.is_fast_track(task):
+        if (manager_utils.is_fast_track(task)
+                and not task.node.disable_power_off):
             LOG.debug('Forcing power off on node %s for a clean reboot into '
                       'the agent image', task.node)
             manager_utils.node_power_action(task, states.POWER_OFF)
         prepare_agent_boot(task)
 
     manager_utils.node_power_action(task, states.REBOOT, timeout)
-    return get_async_step_return_state(task.node)
+    return async_steps.get_return_state(task.node)
+
+
+def step_error_handler(task, logmsg, errmsg=None):
+    """Run the correct handler for the current step.
+
+    :param task: a TaskManager instance.
+    :param logmsg: Message to be logged.
+    :param errmsg: Message for the user. Optional, if not provided `logmsg` is
+        used.
+    """
+    if task.node.provision_state in [states.CLEANING, states.CLEANWAIT]:
+        manager_utils.cleaning_error_handler(task, logmsg, errmsg=errmsg)
+    elif task.node.provision_state in [states.DEPLOYING, states.DEPLOYWAIT]:
+        manager_utils.deploying_error_handler(task, logmsg, errmsg=errmsg)
+    elif task.node.provision_state in [states.SERVICING, states.SERVICEWAIT]:
+        manager_utils.servicing_error_handler(task, logmsg, errmsg=errmsg)
 
 
 def get_root_device_for_deploy(node):
@@ -1748,7 +1698,7 @@ def get_root_device_for_deploy(node):
         source = 'instance_info'
 
     try:
-        return il_utils.parse_root_device_hints(hints)
+        return utils.parse_root_device_hints(hints)
     except ValueError as e:
         raise exception.InvalidParameterValue(
             _('Failed to validate the root device hints %(hints)s (from the '
