@@ -37,8 +37,6 @@ import tempfile
 import time
 
 from eventlet.green import subprocess as green_subprocess
-from ironic_lib import metrics_utils
-from ironic_lib import utils as ironic_utils
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -47,6 +45,7 @@ from oslo_utils import strutils
 from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import task_manager
@@ -71,9 +70,9 @@ REQUIRED_PROPERTIES = {
 }
 OPTIONAL_PROPERTIES = {
     'ipmi_password': _("password. Optional."),
-    'ipmi_hex_kg_key': _('Kg key for IPMIv2 authentication. '
-                         'The key is expected in hexadecimal format. '
-                         'Optional.'),
+    'ipmi_hex_kg_key': _("Kg key for IPMIv2 authentication. "
+                         "The key is expected in hexadecimal format. "
+                         "Optional."),
     'ipmi_port': _("remote IPMI RMCP port. Optional."),
     'ipmi_priv_level': _("privilege level; default is ADMINISTRATOR. One of "
                          "%s. Optional.") % ', '.join(VALID_PRIV_LEVELS),
@@ -206,7 +205,7 @@ def _vendor_aware_boot_device_map(task):
         # NOTE(TheJulia): Similar differences may exist with Cisco UCS
         # hardware when using IPMI, however at present we don't know
         # what the setting would be.
-        # NOTE(TheJulia) I've observed "mc info" manufacter name of a cisco
+        # NOTE(TheJulia) I've observed "mc info" manufacturer name of a cisco
         # c-series machine to return "Unknown (0x168B)"
     return boot_dev_map
 
@@ -280,37 +279,42 @@ def _console_pwfile_path(uuid):
 
 
 @contextlib.contextmanager
-def _make_password_file(password):
-    """Makes a temporary file that contains the password.
+def _prepare_ipmi_password(driver_info):
+    """Prepares the IPMI password by either setting it in the environment
 
-    :param password: the password
-    :returns: the absolute pathname of the temporary file
+    or creating a temporary file.
+
+    :param driver_info: the ipmitool parameters for accessing a node.
+    :returns: the absolute pathname of the password
     :raises: PasswordFileFailedToCreate from creating or writing to the
              temporary file
     """
-    f = None
-    try:
-        f = tempfile.NamedTemporaryFile(mode='w', dir=CONF.tempdir)
-        f.write(str(password))
-        f.flush()
-    except (IOError, OSError) as exc:
-        if f is not None:
-            f.close()
-        raise exception.PasswordFileFailedToCreate(error=exc)
-    except Exception:
-        with excutils.save_and_reraise_exception():
+    if CONF.ipmi.store_cred_in_env:
+        yield _persist_ipmi_password(driver_info)
+    else:
+        f = None
+        try:
+            f = tempfile.NamedTemporaryFile(mode='w', dir=CONF.tempdir)
+            f.write(str(driver_info['password'] or '\0'))
+            f.flush()
+        except (IOError, OSError) as exc:
             if f is not None:
                 f.close()
+            raise exception.PasswordFileFailedToCreate(error=exc)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                if f is not None:
+                    f.close()
 
-    try:
-        # NOTE(jlvillal): This yield can not be in the try/except block above
-        # because an exception by the caller of this function would then get
-        # changed to a PasswordFileFailedToCreate exception which would mislead
-        # about the problem and its cause.
-        yield f.name
-    finally:
-        if f is not None:
-            f.close()
+        try:
+            # NOTE(jlvillal): This yield can not be in the try/except block
+            # above because an exception by the caller of this function would
+            # then get changed to a PasswordFileFailedToCreate exception which
+            # would mislead about the problem and its cause.
+            yield ('-f', f.name)
+        finally:
+            if f is not None:
+                f.close()
 
 
 def is_bridging_enabled(node):
@@ -495,6 +499,25 @@ def _get_ipmitool_args(driver_info, pw_file=None):
     return args
 
 
+def _persist_ipmi_password(driver_info):
+    """Persists IPMI password for passing to the ipmitool
+
+    :param driver_info: driver info with the ipmitool parameters
+    """
+    env = {}
+    if CONF.ipmi.store_cred_in_env:
+        password = driver_info.get('password')
+        if password:
+            env = {'IPMI_PASSWORD': password}
+        return '-E', env
+
+    path = _console_pwfile_path(driver_info['uuid'])
+    pw_file = console_utils.make_persistent_password_file(
+        path, driver_info['password'] or '\0')
+
+    return '-f', pw_file
+
+
 def _ipmitool_timing_args():
     if not _is_option_supported('timing'):
         return []
@@ -524,7 +547,7 @@ def _ipmitool_timing_args():
 
 
 def choose_cipher_suite(actual_cipher_suite):
-    """Gives the possible next avaible cipher suite version.
+    """Gives the possible next available cipher suite version.
 
     Based on CONF.ipmi.cipher_suite_versions and the last cipher suite version
     used that failed. This function is only called if the node doesn't have
@@ -560,9 +583,11 @@ def check_cipher_suite_errors(cmd_stderr):
     :returns: True if the cmd_stderr contains a cipher suite error,
         False otherwise.
     """
-    cs_errors = ["Unsupported cipher suite ID",
-                 "Error in open session response message :"
-                 " no matching cipher suite"]
+    cs_errors = [
+        "Unsupported cipher suite ID",
+        "Error in open session response message : no matching cipher suite",
+        "Error in open session response message : invalid role",
+    ]
     for cs_err in cs_errors:
         if cmd_stderr is not None and cs_err in cmd_stderr:
             return True
@@ -644,10 +669,15 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None,
         # 'ipmitool' command will prompt password if there is no '-f'
         # option, we set it to '\0' to write a password file to support
         # empty password
-        with _make_password_file(driver_info['password'] or '\0') as pw_file:
-            cmd_args.append('-f')
-            cmd_args.append(pw_file)
+
+        with _prepare_ipmi_password(driver_info) as (flag, env_path):
+            cmd_args.append(flag)
+            if CONF.ipmi.store_cred_in_env:
+                extra_args['env_variables'] = env_path
+            else:
+                cmd_args.append(env_path)
             cmd_args.extend(command.split(" "))
+
             try:
                 out, err = utils.execute(*cmd_args, **extra_args)
                 return out, err
@@ -679,6 +709,7 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None,
                                     'Error: %(error)s',
                                     {'node': driver_info['uuid'],
                                      'cmd': e.cmd, 'error': e})
+
             finally:
                 LAST_CMD_TIME[driver_info['address']] = time.time()
 
@@ -709,6 +740,9 @@ def _set_and_wait(task, power_action, driver_info, timeout=None):
         cmd_name = "soft"
         target_state = states.POWER_OFF
         timeout = timeout or CONF.conductor.soft_power_off_timeout
+    elif power_action == states.REBOOT:
+        cmd_name = "reset"
+        target_state = states.POWER_ON
 
     # NOTE(sambetts): Retries for ipmi power action failure will be handled by
     # the _exec_ipmitool function, so no need to wrap this call in its own
@@ -725,6 +759,10 @@ def _set_and_wait(task, power_action, driver_info, timeout=None):
                     "with error: %(error)s.",
                     {'node_id': driver_info['uuid'], 'cmd': cmd, 'error': e})
         raise exception.IPMIFailure(cmd=cmd)
+    if power_action == states.REBOOT:
+        LOG.debug('Sleeping 15 seconds to give the node %s a chance to power '
+                  'off before checking its state', task.node.uuid)
+        time.sleep(15)
     return cond_utils.node_wait_for_power_state(task, target_state,
                                                 timeout=timeout)
 
@@ -1054,6 +1092,11 @@ class IPMIPower(base.PowerInterface):
         elif power_state == states.SOFT_POWER_OFF:
             _soft_power_off(task, driver_info, timeout=timeout)
         elif power_state == states.SOFT_REBOOT:
+            if task.node.disable_power_off:
+                # There is no way to implement this in ipmitool, apparently
+                raise exception.UnsupportedHardwareFeature(
+                    node=task.node.uuid,
+                    feature=states.SOFT_POWER_OFF)
             _soft_power_off(task, driver_info, timeout=timeout)
             driver_utils.ensure_next_boot_device(task, driver_info)
             _power_on(task, driver_info, timeout=timeout)
@@ -1079,6 +1122,10 @@ class IPMIPower(base.PowerInterface):
 
         """
         driver_info = _parse_driver_info(task.node)
+        if task.node.disable_power_off:
+            driver_utils.ensure_next_boot_device(task, driver_info)
+            _set_and_wait(task, states.REBOOT, driver_info, timeout=timeout)
+            return
         # NOTE(jlvillal): Some BMCs will error if setting power state to off if
         # the node is already turned off.
         current_status = _power_status(driver_info)
@@ -1209,8 +1256,8 @@ class IPMIManagement(base.ManagementInterface):
         boot_mode = boot_mode_utils.get_boot_mode(task.node)
         if boot_mode == 'uefi':
             # Long story short: UEFI was added to IPMI after the final spec
-            # release occured. This means BMCs may actually NEED to be
-            # explicitly told if the boot is persistant because the
+            # release occurred. This means BMCs may actually NEED to be
+            # explicitly told if the boot is persistent because the
             # BMC may return a value which is explicitly parsed as
             # no change, BUT the BMC may treat that as operational default.
             efi_persistence = '0xe0' if persistent else '0xa0'
@@ -1340,7 +1387,7 @@ class IPMIManagement(base.ManagementInterface):
         """
         driver_info = _parse_driver_info(task.node)
         # with '-v' option, we can get the entire sensor data including the
-        # extended sensor informations
+        # extended sensor information
         cmd = "sdr -v"
         try:
             out, err = _exec_ipmitool(
@@ -1390,6 +1437,8 @@ class VendorPassthru(base.VendorInterface):
     }
 
     @METRICS.timer('VendorPassthru.send_raw')
+    @base.service_step(priority=0,
+                       argsinfo=_send_raw_step_args)
     @base.deploy_step(priority=0,
                       argsinfo=_send_raw_step_args)
     @base.clean_step(priority=0, abortable=False,
@@ -1541,17 +1590,17 @@ class IPMIConsole(base.ConsoleInterface):
                  created
         :raises: ConsoleSubprocessFailed when invoking the subprocess failed
         """
-        path = _console_pwfile_path(driver_info['uuid'])
-        pw_file = console_utils.make_persistent_password_file(
-            path, driver_info['password'] or '\0')
-        ipmi_cmd = self._get_ipmi_cmd(driver_info, pw_file)
-        ipmi_cmd += ' sol activate'
-
         try:
-            start_method(driver_info['uuid'], driver_info['port'], ipmi_cmd)
-        except (exception.ConsoleError, exception.ConsoleSubprocessFailed):
-            with excutils.save_and_reraise_exception():
-                ironic_utils.unlink_without_raise(path)
+            cmd = self._get_ipmi_cmd(driver_info, pw_file=None)
+            cmd += ' sol activate'
+            start_method(driver_info['uuid'], driver_info['port'], cmd)
+        except (exception.ConsoleError,
+                exception.ConsoleSubprocessFailed) as e:
+            LOG.exception('IPMI Error while attempting "%(cmd)s" '
+                          'for node %(node)s. Error: %(error)s',
+                          {'node': driver_info['uuid'],
+                           'cmd': cmd, 'error': e})
+            raise
 
 
 class IPMIShellinaboxConsole(IPMIConsole):
@@ -1610,8 +1659,7 @@ class IPMIShellinaboxConsole(IPMIConsole):
         try:
             console_utils.stop_shellinabox_console(task.node.uuid)
         finally:
-            ironic_utils.unlink_without_raise(
-                _console_pwfile_path(task.node.uuid))
+            utils.unlink_without_raise(_console_pwfile_path(task.node.uuid))
         _release_allocated_port(task)
 
     def _exec_stop_console(self, driver_info):
@@ -1622,6 +1670,14 @@ class IPMIShellinaboxConsole(IPMIConsole):
     def get_console(self, task):
         """Get the type and connection information about the console."""
         driver_info = _parse_driver_info(task.node)
+
+        try:
+            self._exec_stop_console(driver_info)
+        except OSError:
+            # We need to drop any existing sol sessions with sol deactivate.
+            # OSError is raised when sol session is already deactivated,
+            # so we can ignore it.
+            pass
         url = console_utils.get_shellinabox_console_url(driver_info['port'])
         return {'type': 'shellinabox', 'url': url}
 
@@ -1669,8 +1725,7 @@ class IPMISocatConsole(IPMIConsole):
         try:
             console_utils.stop_socat_console(task.node.uuid)
         finally:
-            ironic_utils.unlink_without_raise(
-                _console_pwfile_path(task.node.uuid))
+            utils.unlink_without_raise(_console_pwfile_path(task.node.uuid))
         self._exec_stop_console(driver_info)
         _release_allocated_port(task)
 
@@ -1685,5 +1740,13 @@ class IPMISocatConsole(IPMIConsole):
         :param task: a task from TaskManager
         """
         driver_info = _parse_driver_info(task.node)
+
+        try:
+            self._exec_stop_console(driver_info)
+        except OSError:
+            # We need to drop any existing sol sessions with sol deactivate.
+            # OSError is raised when sol session is already deactivated,
+            # so we can ignore it.
+            pass
         url = console_utils.get_socat_console_url(driver_info['port'])
         return {'type': 'socat', 'url': url}
