@@ -17,7 +17,6 @@ import collections
 import time
 from urllib.parse import urlparse
 
-from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import timeutils
 import sushy
@@ -28,6 +27,7 @@ from ironic.common import components
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import indicator_states
+from ironic.common import metrics_utils
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import periodics
@@ -1340,6 +1340,18 @@ class RedfishManagement(base.ManagementInterface):
             LOG.error(msg)
             raise exception.RedfishError(error=msg)
 
+    @task_manager.require_exclusive_lock
+    def get_virtual_media(self, task):
+        """Get all virtual media devices from the node.
+
+            :param task: A task from TaskManager.
+
+        """
+        LOG.info('Called redfish.management.get_virtual_media,'
+                 'for the node=%(node)s',
+                 {'node': task.node.uuid})
+        return redfish_boot.get_vmedia(task)
+
     @task_manager.require_exclusive_lock
     def attach_virtual_media(self, task, device_type, image_url):
         """Attach a virtual media device to the node.
